// Aer Fixus' 3DO Controller Tester
// Built upon the framework set down by Rastersoft (http://www.rastersoft.net)
// 
// Released free of any licenses
// Credit would be appreciated, but not required.

#include "../../../../3DODev/includes/controlpad.h"
#include "../../../../3DODev/includes/debug3do.h"
#include "../../../../3DODev/includes/event.h"

#include "Game.h"
#include "Display.h"
#include "Sprite.h"
#include "System.h"

// Controller index array
uint32	gController[8];

//currently unused. I figured it would be useful, but never actually used it...
enum Buttons {A, B, C, LEFT, RIGHT, UP, DOWN, START, SELECT, LEFT_BUMPER, RIGHT_BUMPER} ;

//we define our max controllers.
//dropping this will actually change how many controllers we check for
int numMaxControllers = 8;

//why I need to do this, I have no idea.
//it yelled at me if I initialized it in a function...
int i = 0;

// Background image
static char*	sBackgroundImage	= "controllerbase.imag";

// buttons animations (we are only using one frame at a time)
static char		*sButtonsFace		= "buttonsFace.anim",
				*sButtonsCenter		= "buttonsCenter.anim",
				*sButtonsLR			= "buttonsLeftRight.anim",
				*sButtonsUD			= "buttonsUpDown.anim",
				*sButtonsLB			= "buttonsShiftLeft.anim",
				*sButtonsRB			= "buttonsShiftRight.anim";
//sprite indexes
int	gButtonASprite,
	gButtonBSprite,
	gButtonCSprite,
	gButtonUSprite,
	gButtonDSprite,
	gButtonLSprite,
	gButtonRSprite,
	gButtonLBSprite,
	gButtonRBSprite,
	gButtonSelectSprite,
	gButtonStartSprite;

bool InitializeGame()
{
	PRT(("Initializing the game\n"));
	
	// Load a background image
	if (false == LoadBackground(sBackgroundImage))
		return	false;
			
	// Initialize the sprites
	InitializeSprites();
	
	//create the button sprites
	createButtons();
	
	return	true;
}

void createButtons()
{
	// Add button sprites
	gButtonLBSprite		= AddSprite(sButtonsLB);
	gButtonRBSprite		= AddSprite(sButtonsRB);
	
	gButtonUSprite		= AddSprite(sButtonsUD);
	gButtonDSprite		= AddSprite(sButtonsUD);
	gButtonLSprite		= AddSprite(sButtonsLR);
	gButtonRSprite		= AddSprite(sButtonsLR);
	
	gButtonSelectSprite	= AddSprite(sButtonsCenter);
	gButtonStartSprite	= AddSprite(sButtonsCenter);
	
	gButtonASprite		= AddSprite(sButtonsFace);
	gButtonBSprite		= AddSprite(sButtonsFace);
	gButtonCSprite		= AddSprite(sButtonsFace);

	//set the positions for each button
	SetSpritePosition(gButtonASprite, 206, 153);
	SetSpritePosition(gButtonBSprite, 225, 136);
	SetSpritePosition(gButtonCSprite, 247, 124);
	
	SetSpritePosition(gButtonUSprite, 81, 126);
	SetSpritePosition(gButtonDSprite, 81, 151);
	SetSpritePosition(gButtonLSprite, 66, 141);
	SetSpritePosition(gButtonRSprite, 91, 141);
	
	SetSpritePosition(gButtonSelectSprite, 132, 145);
	SetSpritePosition(gButtonStartSprite, 170, 145);
	
	SetSpritePosition(gButtonRBSprite, 195, 96);
	SetSpritePosition(gButtonLBSprite, 67, 95);
	
	//set the sprites as visible
	SetSpriteVisible(gButtonASprite, true);
	SetSpriteVisible(gButtonBSprite, true);
	SetSpriteVisible(gButtonCSprite, true);
	
	SetSpriteVisible(gButtonUSprite, true);
	SetSpriteVisible(gButtonDSprite, true);
	SetSpriteVisible(gButtonLSprite, true);
	SetSpriteVisible(gButtonRSprite, true);
	
	SetSpriteVisible(gButtonSelectSprite, true);
	SetSpriteVisible(gButtonStartSprite, true);
	
	SetSpriteVisible(gButtonLBSprite, true);
	SetSpriteVisible(gButtonRBSprite, true);

	//make sure that all of our sprites are set to 0, AKA the blank frame
	resetButtonSprites();
}

void resetButtonSprites()
{
	//set frame to 0 (the empty frame)
	SetSpriteFrame(gButtonASprite, 0);
	SetSpriteFrame(gButtonBSprite, 0);
	SetSpriteFrame(gButtonCSprite, 0);
	
	SetSpriteFrame(gButtonUSprite, 0);
	SetSpriteFrame(gButtonDSprite, 0);
	SetSpriteFrame(gButtonLSprite, 0);
	SetSpriteFrame(gButtonRSprite, 0);
	
	SetSpriteFrame(gButtonSelectSprite, 0);
	SetSpriteFrame(gButtonStartSprite, 0);
	
	SetSpriteFrame(gButtonLBSprite, 0);
	SetSpriteFrame(gButtonRBSprite, 0);
}

bool GameLoop()
{
	//this is where we check our input and set what color each button should be
	checkInput();

	//SOMETHING IS RESETTING MY X TO 0!
	//FIND IT AND DESTROY IT
	//until then, here's a messy fix for that issue
	SetSpritePosition(gButtonLBSprite, 67, 95);
	
	Update();
	
	return	false;
}

void checkInput()
{
	//reset all our sprites to blank
	//if a button doesn't get pressed, they stay blank
	resetButtonSprites();
	
	//count DOWN from max controller num
	//that will give lowered number controllers priority
	//we need a priority because we can't show more than one color at once!
	for (i = numMaxControllers; i >= 1; i--)
	{
		readControlPad(i);
		
		if (gController[i] & ControlA)
			SetSpriteFrame(gButtonASprite, i);
		
		if (gController[i] & ControlB)
			SetSpriteFrame(gButtonBSprite, i);
		
		if (gController[i] & ControlC)
			SetSpriteFrame(gButtonCSprite, i);
		
		if (gController[i] & ControlUp)
			SetSpriteFrame(gButtonUSprite, i);
		
		if (gController[i] & ControlDown)
			SetSpriteFrame(gButtonDSprite, i);
		
		if (gController[i] & ControlLeft)
			SetSpriteFrame(gButtonLSprite, i);
		
		if (gController[i] & ControlRight)
			SetSpriteFrame(gButtonRSprite, i);
		
		if (gController[i] & ControlLeftShift)
			SetSpriteFrame(gButtonLBSprite, i);
		
		if (gController[i] & ControlRightShift)
			SetSpriteFrame(gButtonRBSprite, i);
		
		if (gController[i] & ControlStart)
			SetSpriteFrame(gButtonStartSprite, i);
		
		if (gController[i] & ControlX)
			SetSpriteFrame(gButtonSelectSprite, i);
	}
}

void readControlPad(int controllerNum)
{
 	//Read the pad we are told to read
	//it will be read into an array of uint32, indexing our 8 controllers
	int	iStatus	= DoControlPad(controllerNum, &gController[i], (ControlUp | ControlDown | ControlLeft | ControlRight | ControlA | ControlB | ControlC | ControlStart | ControlX | ControlLeftShift | ControlRightShift));
}
