// RasterSoft Invades 3DO demo by Charles Doty/RasterSoft
// http://www.rastersoft.net
// Released free of any licenses
// Credit would be appreciated, but not required.

#include "Sprite.h"
#include "System.h"

#include "../../../../3DODev/includes/celutils.h"
#include "../../../../3DODev/includes/debug3do.h"
#include "../../../../3DODev/includes/displayutils.h"
#include "../../../../3DODev/includes/mem.h"

// Sprite list
sSprite	SpriteList[MAX_SPRITES];

// Screen context
extern	ScreenContext	*gScreenContext;

void InitializeSprites()
{
	int	iLoop;
	
	PRT(("Initialize sprites\n"));

	// Set all sprites to unused
	for (iLoop = 0; iLoop < MAX_SPRITES; ++iLoop)
	{
		SpriteList[iLoop].pAnim	= NULL;
		SpriteList[iLoop].bUsed	= false;
		SpriteList[iLoop].bShow	= false;
	}
}

void UpdateSprites(int iCurrentScreen)
{
	int	iLoop;
	
	Point	ptQuad[4];

	Item	itmBitmapItems	= gScreenContext->sc_BitmapItems[iCurrentScreen];
	
	// Draw each cel
	for (iLoop = 0; iLoop < MAX_SPRITES; ++iLoop)
	{
		if (true == SpriteList[iLoop].bUsed && SpriteList[iLoop].pAnim	!= NULL && true == SpriteList[iLoop].bShow)
		{
			int	iX	= SpriteList[iLoop].ptPosition.pt_X;
			int	iY	= SpriteList[iLoop].ptPosition.pt_Y;

			CCB	*pCCB	= SpriteList[iLoop].pCCB;
			
			pCCB	= GetAnimCel(SpriteList[iLoop].pAnim, 0);
			
			ptQuad[eTopLeft].pt_X		= iX +						SpriteList[iLoop].vert.ul_x;
			ptQuad[eTopLeft].pt_Y		= iY +						SpriteList[iLoop].vert.ul_y;
			ptQuad[eTopRight].pt_X		= iX + pCCB->ccb_Width +	SpriteList[iLoop].vert.ur_x;
			ptQuad[eTopRight].pt_Y		= iY +						SpriteList[iLoop].vert.ur_y;
			
			ptQuad[eBottomLeft].pt_X	= iX +						SpriteList[iLoop].vert.ll_x;
			ptQuad[eBottomLeft].pt_Y	= iY + pCCB->ccb_Height +	SpriteList[iLoop].vert.ll_y;
			ptQuad[eBottomRight].pt_X	= iX + pCCB->ccb_Width +	SpriteList[iLoop].vert.lr_x;
			ptQuad[eBottomRight].pt_Y	= iY + pCCB->ccb_Height +	SpriteList[iLoop].vert.lr_y;

			//this can transform the sprite into a 3d quad
			MapCel(pCCB, ptQuad);
			LAST_CEL(pCCB);

			PRT(("Drawing cel %d %d %d\n", iLoop, iX, iY));
			
			//this actually draws the sprite onto the bitmap buffer
			DrawCels(itmBitmapItems, pCCB);
		}
	}
}

int	AddSprite(char *szCelFile)
{
	char	szFilename[128];
	
	// Find the next available sprite
	int	iSpriteIndex	= FindAvailableSprite();

	if (iSpriteIndex >= 0)
	{
		SpriteList[iSpriteIndex].bUsed	= true;

		GetArtFilename(szCelFile, szFilename);
			
		PRT(("Add sprite %s\n", szFilename));

		// Setup the animation
		SpriteList[iSpriteIndex].pAnim	= LoadAnim(szFilename, MEMTYPE_CEL);
	
		if (NULL == SpriteList[iSpriteIndex].pAnim)
		{
			PRT(("Unable to load animation %s\n", szFilename));
		
			return	-1;
		}

		SpriteList[iSpriteIndex].pCCB	= SetCommonCCB(SpriteList[iSpriteIndex].pAnim);
	}

	return	iSpriteIndex;
}

void RemoveSprite(int iIndex)
{
	// Remove a sprite by setting it to unused.
	SpriteList[iIndex].bUsed	= false;

	// Unload the animation
	if (SpriteList[iIndex].pAnim != NULL)
	{
		UnloadAnim(SpriteList[iIndex].pAnim);
		
		SpriteList[iIndex].pAnim	= NULL;
	}
}

void SetSpritePosition(int iIndex, int iX, int iY)
{
	SpriteList[iIndex].ptPosition.pt_X	= iX;
	SpriteList[iIndex].ptPosition.pt_Y	= iY;
}

void SetSpriteVisible(int iIndex, bool bVisible)
{
	SpriteList[iIndex].bShow	= bVisible;
}

void SetSpriteFrame(int iIndex, int iFrame)
{
	SpriteList[iIndex].pAnim->cur_Frame	= Convert32_F16(iFrame);
}

int FindAvailableSprite()
{
	int	iLoop;
	
	for (iLoop = 0; iLoop < MAX_SPRITES; ++iLoop)
	{
		if (false == SpriteList[iLoop].bUsed)
		{
			return	iLoop;
		}
	}

	PRT(("Unable to find an available sprite\n"));
	
	return	-1;
}

CCB *SetCommonCCB(ANIM* pAnim)
{
	int		iFrameIndex;
	CCB*	pCCB	= pAnim->pentries[0].af_CCB;
	
	for (iFrameIndex = 0; iFrameIndex < pAnim->num_Frames; iFrameIndex++)
	{
		pAnim->pentries[iFrameIndex].af_CCB	= pCCB;
	}

	return	pCCB;
}

int GetSpriteWidth(int iIndex)
{
	return	SpriteList[iIndex].pCCB->ccb_Width;
}

int GetSpriteHeight(int iIndex)
{
	return	SpriteList[iIndex].pCCB->ccb_Height;
}

void setVerticies(int iIndex, 
				  int32 ul_x, int32 ul_y,
				  int32 ur_x, int32 ur_y,
				  int32 ll_x, int32 ll_y,
				  int32 lr_x, int32 lr_y)
{
	SpriteList[iIndex].vert.ul_x = ul_x;
	SpriteList[iIndex].vert.ul_y = ul_y;
	SpriteList[iIndex].vert.ur_x = ur_x;
	SpriteList[iIndex].vert.ur_y = ur_y;
	SpriteList[iIndex].vert.ll_x = ll_x;
	SpriteList[iIndex].vert.ll_y = ll_y;
	SpriteList[iIndex].vert.lr_x = lr_x;
	SpriteList[iIndex].vert.lr_y = lr_y;
}

void addToVerticies(int iIndex, 
				  int32 ul_x, int32 ul_y,
				  int32 ur_x, int32 ur_y,
				  int32 ll_x, int32 ll_y,
				  int32 lr_x, int32 lr_y)
{
	SpriteList[iIndex].vert.ul_x += ul_x;
	SpriteList[iIndex].vert.ul_y += ul_y;
	SpriteList[iIndex].vert.ur_x += ur_x;
	SpriteList[iIndex].vert.ur_y += ur_y;
	SpriteList[iIndex].vert.ll_x += ll_x;
	SpriteList[iIndex].vert.ll_y += ll_y;
	SpriteList[iIndex].vert.lr_x += lr_x;
	SpriteList[iIndex].vert.lr_y += lr_y;
}
/*
void addToUL(int iIndex, int32 ul_x, int32 ul_y)
{
	SpriteList[iIndex].vert.ul_x += ul_x;
	SpriteList[iIndex].vert.ul_y += ul_y;
}
void addToUR(int iIndex, int32 ur_x, int32 ur_y)
{
	SpriteList[iIndex].vert.ur_x += ur_x;
	SpriteList[iIndex].vert.ur_y += ur_y;
}
void addToLL(int iIndex, int32 ll_x, int32 ll_y)
{
	SpriteList[iIndex].vert.ll_x += ll_x;
	SpriteList[iIndex].vert.ll_y += ll_y;
}
void addToLR(int iIndex, int32 lr_x, int32 lr_y)
{
	SpriteList[iIndex].vert.lr_x += lr_x;
	SpriteList[iIndex].vert.lr_y += lr_y;
}

void setUL(int iIndex, int32 ul_x, int32 ul_y)
{
	SpriteList[iIndex].vert.ul_x = ul_x;
	SpriteList[iIndex].vert.ul_y = ul_y;
}
void setUR(int iIndex, int32 ur_x, int32 ur_y)
{
	SpriteList[iIndex].vert.ur_x = ur_x;
	SpriteList[iIndex].vert.ur_y = ur_y;
}
void setLL(int iIndex, int32 ll_x, int32 ll_y)
{
	SpriteList[iIndex].vert.ll_x = ll_x;
	SpriteList[iIndex].vert.ll_y = ll_y;
}
void setLR(int iIndex, int32 lr_x, int32 lr_y)
{
	SpriteList[iIndex].vert.lr_x = lr_x;
	SpriteList[iIndex].vert.lr_y = lr_y;
}*/

/*void SetQuadFromPosAndSize(Point *aQuad, int32 tl_x, int32 tl_y, 
										 int32 tr_x, int32 tr_y, 
										 int32 bl_x, int32 bl_y, 
										 int32 br_x, int32 br_y)
{
	ptQuad.pt_X = tl_x;
	aQuad[kQuadTL].pt_Y = tl_y;

	aQuad[kQuadTR].pt_X = tr_x;
	aQuad[kQuadTR].pt_Y = tr_y;

	aQuad[kQuadBL].pt_X = bl_x;
	aQuad[kQuadBL].pt_Y = bl_y;

	aQuad[kQuadBR].pt_X = br_x;
	aQuad[kQuadBR].pt_Y = br_y;
}*/