// RasterSoft Invades 3DO demo by Charles Doty/RasterSoft
// http://www.rastersoft.net
// Released free of any licenses
// Credit would be appreciated, but not required.

#ifndef __SPRITE_H
#define __SPRITE_H

#include "../../../../3DODev/includes/types.h"
#include "../../../../3DODev/includes/animutils.h"
#include "../../../../3DODev/includes/graphics.h"

// Maximum number of sprites
#define	MAX_SPRITES	12

// Position corners
enum eCorners
{
	eTopLeft,
	eTopRight,
	eBottomRight,
	eBottomLeft
};

typedef struct 
{
	int32 ul_x;
	int32 ul_y;
	int32 ur_x;
	int32 ur_y;
	int32 ll_x;
	int32 ll_y;
	int32 lr_x;
	int32 lr_y;
} sVerticies; 

// Sprite element
typedef struct
{
	// Position
	Point	ptPosition;
	
	// Sprite cel/anim
	ANIM	*pAnim;

	// Sprite CCB
	CCB		*pCCB;
	
	// Is this sprite used
	bool	bUsed;

	// Show the sprite
	bool	bShow;

	// Verticies
	sVerticies vert;
} sSprite;

// Initialize sprites
void InitializeSprites();

// Update sprites
void UpdateSprites(int iCurrentScreen);

// Add sprite
int	AddSprite(char *szCelFile);

// Remove sprite
void RemoveSprite(int iIndex);

// Set sprite position
void SetSpritePosition(int iIndex, int iX, int iY);

// Set sprite frame
void SetSpriteFrame(int iIndex, int iFrame);

// Set sprite visibility
void SetSpriteVisible(int iIndex, bool bVisible);

// Find available sprite
int FindAvailableSprite();

// Set common CCB
CCB *SetCommonCCB(ANIM* pAnim);

// Get sprite width
int GetSpriteWidth(int iIndex);

// Get sprite height
int GetSpriteHeight(int iIndex);

void setVerticies(int iIndex, 
				  int32 ul_x, int32 ul_y,
				  int32 ur_x, int32 ur_y,
				  int32 ll_x, int32 ll_y,
				  int32 lr_x, int32 lr_y);

void addToVerticies(int iIndex, 
				  int32 ul_x, int32 ul_y,
				  int32 ur_x, int32 ur_y,
				  int32 ll_x, int32 ll_y,
				  int32 lr_x, int32 lr_y);

//set quad for one sprite
/*void SetQuadFromPosAndSize(Point *aQuad, int32 tl_x, int32 tl_y, 
										 int32 tr_x, int32 tr_y, 
										 int32 bl_x, int32 bl_y, 
										 int32 br_x, int32 br_y);*/

#endif